/*
 *  vtgprojectmanagerfilteredlistdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerfilteredlistdialog.h"
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include "vtgutils.h"




static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
struct _VtgProjectManagerFilteredListDialogPrivate {
	GtkDialog* _dialog;
	GtkTreeView* _treeview;
	GtkEntry* _entry;
	GtkTreeModelFilter* _model;
	GtkTreeModelSort* _sorted;
	GtkTreeModel* _child_model;
	GPatternSpec* _current_pattern;
};

#define VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_FILTERED_LIST_DIALOG, VtgProjectManagerFilteredListDialogPrivate))
enum  {
	VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_DUMMY_PROPERTY
};
static gboolean _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, GdkEventKey* event, gpointer self);
static gboolean _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self);
static gint _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
static gboolean _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event (GtkTreeView* _sender, GdkEventKey* event, gpointer self);
static void vtg_project_manager_filtered_list_dialog_initialize_ui (VtgProjectManagerFilteredListDialog* self);
static gboolean vtg_project_manager_filtered_list_dialog_filter_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, GtkTreeIter* iter);
static gint vtg_project_manager_filtered_list_dialog_sort_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gpointer vtg_project_manager_filtered_list_dialog_parent_class = NULL;
static void vtg_project_manager_filtered_list_dialog_finalize (GObject* obj);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_warn_if_fail (offset >= 0);
	} else {
		g_warn_if_fail (offset <= string_length);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_warn_if_fail ((offset + len) <= string_length);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectManagerFilteredListDialog* vtg_project_manager_filtered_list_dialog_construct (GType object_type, GtkTreeModel* model) {
	VtgProjectManagerFilteredListDialog * self;
	GtkTreeModel* _tmp1;
	GtkTreeModel* _tmp0;
	g_return_val_if_fail (model != NULL, NULL);
	self = g_object_newv (object_type, 0, NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_child_model = (_tmp1 = (_tmp0 = model, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_child_model == NULL) ? NULL : (self->priv->_child_model = (g_object_unref (self->priv->_child_model), NULL)), _tmp1);
	vtg_project_manager_filtered_list_dialog_initialize_ui (self);
	return self;
}


VtgProjectManagerFilteredListDialog* vtg_project_manager_filtered_list_dialog_new (GtkTreeModel* model) {
	return vtg_project_manager_filtered_list_dialog_construct (VTG_PROJECT_MANAGER_TYPE_FILTERED_LIST_DIALOG, model);
}


static gboolean _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event (GtkEntry* _sender, GdkEventKey* event, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_on_entry_key_press (self, _sender, event);
}


static gboolean _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_filter_model (self, model, iter);
}


static gint _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_sort_model (self, model, a, b);
}


static gboolean _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event (GtkTreeView* _sender, GdkEventKey* event, gpointer self) {
	return vtg_project_manager_filtered_list_dialog_on_treeview_key_press (self, _sender, event);
}


static void vtg_project_manager_filtered_list_dialog_initialize_ui (VtgProjectManagerFilteredListDialog* self) {
	GError * inner_error;
	GtkBuilder* builder;
	GtkDialog* _tmp2;
	GtkDialog* _tmp1;
	GtkTreeView* _tmp4;
	GtkTreeView* _tmp3;
	GtkEntry* _tmp6;
	GtkEntry* _tmp5;
	GtkTreeModelFilter* _tmp7;
	GtkTreeViewColumn* column;
	GtkCellRenderer* renderer;
	GtkTreeModelSort* _tmp8;
	g_return_if_fail (self != NULL);
	inner_error = NULL;
	builder = gtk_builder_new ();
	{
		char* _tmp0;
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			goto __catch21_g_error;
		}
		_tmp0 = (g_free (_tmp0), NULL);
	}
	goto __finally21;
	__catch21_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgprojectmanagerfilteredlistdialog.vala:53: initialize_ui: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally21:
	;
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-db")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
	g_assert (self->priv->_dialog != NULL);
	_tmp4 = NULL;
	_tmp3 = NULL;
	self->priv->_treeview = (_tmp4 = (_tmp3 = GTK_TREE_VIEW (gtk_builder_get_object (builder, "treeview-db-docs")), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3)), (self->priv->_treeview == NULL) ? NULL : (self->priv->_treeview = (g_object_unref (self->priv->_treeview), NULL)), _tmp4);
	g_assert (self->priv->_treeview != NULL);
	_tmp6 = NULL;
	_tmp5 = NULL;
	self->priv->_entry = (_tmp6 = (_tmp5 = GTK_ENTRY (gtk_builder_get_object (builder, "entry-db-filter")), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (self->priv->_entry == NULL) ? NULL : (self->priv->_entry = (g_object_unref (self->priv->_entry), NULL)), _tmp6);
	g_assert (self->priv->_entry != NULL);
	g_signal_connect_object ((GtkWidget*) self->priv->_entry, "key-press-event", (GCallback) _vtg_project_manager_filtered_list_dialog_on_entry_key_press_gtk_widget_key_press_event, self, 0);
	_tmp7 = NULL;
	self->priv->_model = (_tmp7 = (GtkTreeModelFilter*) gtk_tree_model_filter_new (self->priv->_child_model, NULL), (self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL)), _tmp7);
	gtk_tree_model_filter_set_visible_func (self->priv->_model, _vtg_project_manager_filtered_list_dialog_filter_model_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
	column = g_object_ref_sink (gtk_tree_view_column_new ());
	renderer = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ());
	gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, TRUE);
	gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 0);
	gtk_tree_view_append_column (self->priv->_treeview, column);
	_tmp8 = NULL;
	self->priv->_sorted = (_tmp8 = (GtkTreeModelSort*) gtk_tree_model_sort_new_with_model ((GtkTreeModel*) self->priv->_model), (self->priv->_sorted == NULL) ? NULL : (self->priv->_sorted = (g_object_unref (self->priv->_sorted), NULL)), _tmp8);
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_sorted, 0, GTK_SORT_ASCENDING);
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->_sorted, 0, _vtg_project_manager_filtered_list_dialog_sort_model_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
	gtk_tree_view_set_model (self->priv->_treeview, (GtkTreeModel*) self->priv->_sorted);
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (self->priv->_treeview), GTK_SELECTION_SINGLE);
	g_signal_connect_object ((GtkWidget*) self->priv->_treeview, "key-press-event", (GCallback) _vtg_project_manager_filtered_list_dialog_on_treeview_key_press_gtk_widget_key_press_event, self, 0);
	(builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL));
	(column == NULL) ? NULL : (column = (g_object_unref (column), NULL));
	(renderer == NULL) ? NULL : (renderer = (g_object_unref (renderer), NULL));
}


gboolean vtg_project_manager_filtered_list_dialog_run (VtgProjectManagerFilteredListDialog* self) {
	gint result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = gtk_dialog_run (self->priv->_dialog);
	if (result > 0) {
		GtkTreeIter iter = {0};
		if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, &iter)) {
			GtkTreeIter sort = {0};
			gtk_tree_model_sort_convert_iter_to_child_iter (self->priv->_sorted, &sort, &iter);
			gtk_tree_model_filter_convert_iter_to_child_iter (self->priv->_model, &self->selected_iter, &sort);
		} else {
			result = 0;
		}
	}
	gtk_object_destroy ((GtkObject*) self->priv->_dialog);
	return result > 0;
}


gboolean vtg_project_manager_filtered_list_dialog_on_entry_key_press (VtgProjectManagerFilteredListDialog* self, GtkWidget* sender, GdkEventKey* evt) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	_tmp0 = FALSE;
	if (((*evt).state & GDK_MOD1_MASK) == 0) {
		_tmp0 = (*evt).keyval == GDK_Return;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		const char* _tmp1;
		char* filter;
		_tmp1 = NULL;
		filter = (_tmp1 = gtk_entry_get_text (self->priv->_entry), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
		if (vtg_string_utils_is_null_or_empty (filter)) {
			GPatternSpec* _tmp2;
			_tmp2 = NULL;
			self->priv->_current_pattern = (_tmp2 = NULL, (self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL)), _tmp2);
		} else {
			char* _tmp3;
			GPatternSpec* _tmp6;
			_tmp3 = NULL;
			filter = (_tmp3 = vtg_string_utils_replace (filter, " ", "*"), filter = (g_free (filter), NULL), _tmp3);
			if (!g_str_has_suffix (filter, "*")) {
				char* _tmp4;
				_tmp4 = NULL;
				filter = (_tmp4 = g_strconcat (filter, ("*"), NULL), filter = (g_free (filter), NULL), _tmp4);
			}
			if (!g_str_has_prefix (filter, "*")) {
				char* _tmp5;
				_tmp5 = NULL;
				filter = (_tmp5 = g_strconcat ("*", filter, NULL), filter = (g_free (filter), NULL), _tmp5);
			}
			_tmp6 = NULL;
			self->priv->_current_pattern = (_tmp6 = g_pattern_spec_new (filter), (self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL)), _tmp6);
		}
		gtk_tree_model_filter_refilter (self->priv->_model);
		filter = (g_free (filter), NULL);
	}
	return FALSE;
}


gboolean vtg_project_manager_filtered_list_dialog_on_treeview_key_press (VtgProjectManagerFilteredListDialog* self, GtkWidget* sender, GdkEventKey* evt) {
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (sender != NULL, FALSE);
	_tmp0 = FALSE;
	if (((*evt).state & GDK_MOD1_MASK) == 0) {
		_tmp0 = (*evt).keyval == GDK_Return;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (self->priv->_treeview), NULL, NULL)) {
			gtk_dialog_response (self->priv->_dialog, 2);
		}
	}
	return FALSE;
}


static gboolean vtg_project_manager_filtered_list_dialog_filter_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, GtkTreeIter* iter) {
	char* val;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (model != NULL, FALSE);
	if (self->priv->_current_pattern == NULL) {
		return TRUE;
	}
	val = NULL;
	gtk_tree_model_get (model, &(*iter), 0, &val, -1);
	return (_tmp1 = g_pattern_match_string (self->priv->_current_pattern, val), val = (g_free (val), NULL), _tmp1);
}


static gint vtg_project_manager_filtered_list_dialog_sort_model (VtgProjectManagerFilteredListDialog* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	char* vala;
	char* valb;
	gboolean _tmp0;
	gboolean _tmp2;
	gint _tmp4;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (model != NULL, 0);
	vala = NULL;
	valb = NULL;
	gtk_tree_model_get (model, &(*a), 0, &vala, -1);
	gtk_tree_model_get (model, &(*b), 0, &valb, -1);
	_tmp0 = FALSE;
	if (g_str_has_suffix (vala, ".vala")) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = g_str_has_suffix (vala, ".vapi");
	}
	if (_tmp0) {
		char* _tmp1;
		_tmp1 = NULL;
		vala = (_tmp1 = string_substring (vala, (glong) 0, string_get_length (vala) - 5), vala = (g_free (vala), NULL), _tmp1);
	}
	_tmp2 = FALSE;
	if (g_str_has_suffix (valb, ".vala")) {
		_tmp2 = TRUE;
	} else {
		_tmp2 = g_str_has_suffix (valb, ".vapi");
	}
	if (_tmp2) {
		char* _tmp3;
		_tmp3 = NULL;
		valb = (_tmp3 = string_substring (valb, (glong) 0, string_get_length (valb) - 5), valb = (g_free (valb), NULL), _tmp3);
	}
	return (_tmp4 = strcmp (vala, valb), vala = (g_free (vala), NULL), valb = (g_free (valb), NULL), _tmp4);
}


static void vtg_project_manager_filtered_list_dialog_class_init (VtgProjectManagerFilteredListDialogClass * klass) {
	vtg_project_manager_filtered_list_dialog_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerFilteredListDialogPrivate));
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_filtered_list_dialog_finalize;
}


static void vtg_project_manager_filtered_list_dialog_instance_init (VtgProjectManagerFilteredListDialog * self) {
	self->priv = VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG_GET_PRIVATE (self);
	self->priv->_current_pattern = NULL;
}


static void vtg_project_manager_filtered_list_dialog_finalize (GObject* obj) {
	VtgProjectManagerFilteredListDialog * self;
	self = VTG_PROJECT_MANAGER_FILTERED_LIST_DIALOG (obj);
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
	(self->priv->_treeview == NULL) ? NULL : (self->priv->_treeview = (g_object_unref (self->priv->_treeview), NULL));
	(self->priv->_entry == NULL) ? NULL : (self->priv->_entry = (g_object_unref (self->priv->_entry), NULL));
	(self->priv->_model == NULL) ? NULL : (self->priv->_model = (g_object_unref (self->priv->_model), NULL));
	(self->priv->_sorted == NULL) ? NULL : (self->priv->_sorted = (g_object_unref (self->priv->_sorted), NULL));
	(self->priv->_child_model == NULL) ? NULL : (self->priv->_child_model = (g_object_unref (self->priv->_child_model), NULL));
	(self->priv->_current_pattern == NULL) ? NULL : (self->priv->_current_pattern = (g_pattern_spec_free (self->priv->_current_pattern), NULL));
	G_OBJECT_CLASS (vtg_project_manager_filtered_list_dialog_parent_class)->finalize (obj);
}


static GType vtg_project_manager_filtered_list_dialog_type_id = 0;
GType vtg_project_manager_filtered_list_dialog_get_type (void) {
	return vtg_project_manager_filtered_list_dialog_type_id;
}


GType vtg_project_manager_filtered_list_dialog_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerFilteredListDialogClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_filtered_list_dialog_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerFilteredListDialog), 0, (GInstanceInitFunc) vtg_project_manager_filtered_list_dialog_instance_init, NULL };
	vtg_project_manager_filtered_list_dialog_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerFilteredListDialog", &g_define_type_info, 0);
	return vtg_project_manager_filtered_list_dialog_type_id;
}




