/*
 *  vtgsymbolcompletionprovider.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgsymbolcompletionprovider.h"
#include <vscsourcebuffer.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtksourcecompletion/gsc-info.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-statusbar.h>
#include <gtk/gtk.h>
#include <gedit/gedit-document.h>
#include <vscparsermanager.h>
#include <gdk/gdk.h>
#include <vscsymbolcompletionitem.h>
#include <vscsymbolcompletionresult.h>
#include <vala/valadatatype.h>
#include <vscsymbolcompletionfilteroptions.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>
#include <gtksourcecompletion/gsc-utils.h>
#include <gdk/gdkkeysyms.h>
#include <gtksourcecompletion/gsc-proposal.h>
#include <gee/list.h>
#include "vtgsymbolcompletiontrigger.h"
#include "vtgutils.h"




struct _VtgSymbolCompletionProviderPrivate {
	VscSourceBuffer* _sb;
	VscSymbolCompletion* _completion;
	GeditView* _view;
	GList* list;
	GdkPixbuf* _icon_generic;
	GdkPixbuf* _icon_field;
	GdkPixbuf* _icon_method;
	GdkPixbuf* _icon_class;
	GdkPixbuf* _icon_struct;
	GdkPixbuf* _icon_property;
	GdkPixbuf* _icon_signal;
	GdkPixbuf* _icon_iface;
	GdkPixbuf* _icon_const;
	GdkPixbuf* _icon_enum;
	GdkPixbuf* _icon_namespace;
	guint timeout_id;
	gboolean all_doc;
	gint prealloc_index;
	gint counter;
	gboolean cache_building;
	gboolean prev_cache_building;
	gboolean tooltip_is_visible;
	VtgSymbolCompletionTrigger* _last_trigger;
	guint sb_msg_id;
	guint sb_context_id;
	VtgPlugin* _plugin;
	GscInfo* _calltip_window;
	gboolean need_parse;
	gint current_edited_line;
};

#define VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProviderPrivate))
enum  {
	VTG_SYMBOL_COMPLETION_PROVIDER_DUMMY_PROPERTY,
	VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION,
	VTG_SYMBOL_COMPLETION_PROVIDER_VIEW,
	VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN
};
static void _g_list_free_g_object_unref (GList* self);
static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self);
static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self);
static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self);
static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, GdkEventKey* evt);
static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc);
static void vtg_symbol_completion_provider_real_finish (GscProvider* base);
static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base);
static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger);
static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b);
static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon);
static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b);
static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* result);
static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** word, char** line, gint* lineno, gint* colno);
static VscSymbolCompletionItem* vtg_symbol_completion_provider_find_method_signature (VtgSymbolCompletionProvider* self);
static VscSymbolCompletionResult* vtg_symbol_completion_provider_find_proposals (VtgSymbolCompletionProvider* self);
static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc);
static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value);
static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value);
static void vtg_symbol_completion_provider_set_plugin (VtgSymbolCompletionProvider* self, VtgPlugin* value);
static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, GdkEventKey* event, gpointer self);
static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self);
static void __lambda0 (VscParserManager* sender, VtgSymbolCompletionProvider* self);
static void ___lambda0_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self);
static void __lambda1 (VscParserManager* sender, VtgSymbolCompletionProvider* self);
static void ___lambda1_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self);
static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_symbol_completion_provider_parent_class = NULL;
static GscProviderIface* vtg_symbol_completion_provider_gsc_provider_parent_iface = NULL;
static void vtg_symbol_completion_provider_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);



static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_construct (GType object_type, VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSymbolCompletionProvider * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	g_return_val_if_fail (completion != NULL, NULL);
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	__params_it->name = "completion";
	g_value_init (&__params_it->value, VSC_TYPE_SYMBOL_COMPLETION);
	g_value_set_object (&__params_it->value, completion);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_new (VtgPlugin* plugin, GeditView* view, VscSymbolCompletion* completion) {
	return vtg_symbol_completion_provider_construct (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, plugin, view, completion);
}


static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self) {
	gboolean _tmp0;
	gboolean _tmp1;
	g_return_val_if_fail (self != NULL, FALSE);
	_tmp0 = FALSE;
	_tmp1 = FALSE;
	if (self->priv->cache_building) {
		_tmp1 = !self->priv->tooltip_is_visible;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		_tmp0 = self->priv->prev_cache_building == FALSE;
	} else {
		_tmp0 = FALSE;
	}
	if (_tmp0) {
		GeditStatusbar* _tmp2;
		GeditStatusbar* status_bar;
		self->priv->prev_cache_building = self->priv->cache_building;
		_tmp2 = NULL;
		status_bar = (_tmp2 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		if (self->priv->sb_msg_id != 0) {
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
		}
		self->priv->sb_msg_id = gtk_statusbar_push ((GtkStatusbar*) status_bar, self->priv->sb_context_id, "rebuilding symbol cache...");
		(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
	} else {
		gboolean _tmp3;
		_tmp3 = FALSE;
		if (self->priv->cache_building == FALSE) {
			_tmp3 = self->priv->prev_cache_building == TRUE;
		} else {
			_tmp3 = FALSE;
		}
		if (_tmp3) {
			GeditStatusbar* _tmp4;
			GeditStatusbar* status_bar;
			self->priv->prev_cache_building = FALSE;
			/*hide tip, show proposal list*/
			_tmp4 = NULL;
			status_bar = (_tmp4 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
			self->priv->sb_msg_id = (guint) 0;
			if (self->priv->_last_trigger != NULL) {
				vtg_symbol_completion_trigger_trigger_event ((VTG_SYMBOL_COMPLETION_TRIGGER (self->priv->_last_trigger)));
			}
			(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
		}
	}
	return FALSE;
}


static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->counter == 0) {
		vtg_symbol_completion_provider_parse (self, GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)));
		self->priv->timeout_id = (guint) 0;
		return FALSE;
	} else {
		self->priv->counter--;
		return TRUE;
	}
}


static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_timeout_parse (self);
}


static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, GdkEventKey* evt) {
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter start = {0};
	gint line;
	gunichar ch;
	gboolean _tmp4;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	line = gtk_text_iter_get_line (&start);
	ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
	if (ch == '(') {
		vtg_symbol_completion_provider_show_calltip (self);
	} else {
		gboolean _tmp0;
		gboolean _tmp1;
		_tmp0 = FALSE;
		_tmp1 = FALSE;
		if ((*evt).keyval == GDK_Escape) {
			_tmp1 = TRUE;
		} else {
			_tmp1 = ch == ')';
		}
		if (_tmp1) {
			_tmp0 = TRUE;
		} else {
			_tmp0 = (*evt).keyval == GDK_Return;
		}
		if (_tmp0) {
			vtg_symbol_completion_provider_hide_calltip (self);
		}
	}
	if (self->priv->counter <= 0) {
		gboolean _tmp2;
		_tmp2 = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp2 = TRUE;
		} else {
			_tmp2 = ch == ';';
		}
		if (_tmp2) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
			/*immediatly (0.1sec)*/
			self->priv->current_edited_line = -1;
		} else {
			if ((*evt).keyval != GDK_Escape) {
				self->priv->need_parse = TRUE;
			}
		}
	} else {
		gboolean _tmp3;
		_tmp3 = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp3 = TRUE;
		} else {
			_tmp3 = ch == ';';
		}
		if (_tmp3) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
		} else {
			/*immediatly (0.1sec)*/
			self->priv->all_doc = FALSE;
			self->priv->counter = 5;
		}
	}
	_tmp4 = FALSE;
	if (self->priv->need_parse) {
		_tmp4 = self->priv->current_edited_line != line;
	} else {
		_tmp4 = FALSE;
	}
	if (_tmp4) {
		self->priv->need_parse = FALSE;
		self->priv->current_edited_line = line;
		self->priv->timeout_id = g_timeout_add ((guint) 25, _vtg_symbol_completion_provider_on_timeout_parse_gsource_func, self);
	}
	return FALSE;
}


static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self) {
	VscSymbolCompletionItem* result;
	g_return_if_fail (self != NULL);
	result = vtg_symbol_completion_provider_find_method_signature (self);
	if (result != NULL) {
		const char* _tmp0;
		char* calltip_text;
		if (self->priv->_calltip_window == NULL) {
			vtg_symbol_completion_provider_initialize_calltip_window (self);
		}
		_tmp0 = NULL;
		calltip_text = (_tmp0 = result->info, (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
		if (calltip_text != NULL) {
			gsc_info_set_markup (self->priv->_calltip_window, calltip_text);
			gsc_info_move_to_cursor (self->priv->_calltip_window, (GtkTextView*) self->priv->_view);
			gtk_widget_show_all ((GtkWidget*) self->priv->_calltip_window);
		}
		calltip_text = (g_free (calltip_text), NULL);
	} else {
		g_debug ("vtgsymbolcompletionprovider.vala:218: calltip no proposal found");
	}
	(result == NULL) ? NULL : (result = (g_object_unref (result), NULL));
}


static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->_calltip_window == NULL) {
		return;
	}
	gtk_widget_hide ((GtkWidget*) self->priv->_calltip_window);
}


static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self) {
	GscInfo* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	self->priv->_calltip_window = (_tmp0 = g_object_ref_sink (gsc_info_new ()), (self->priv->_calltip_window == NULL) ? NULL : (self->priv->_calltip_window = (g_object_unref (self->priv->_calltip_window), NULL)), _tmp0);
	gsc_info_set_info_type (self->priv->_calltip_window, GSC_INFO_VIEW_EXTENDED);
	gtk_window_set_transient_for ((GtkWindow*) self->priv->_calltip_window, (GtkWindow*) vtg_plugin_get_gedit_window (self->priv->_plugin));
	gsc_info_set_adjust_width (self->priv->_calltip_window, TRUE, 400);
	gsc_info_set_adjust_height (self->priv->_calltip_window, TRUE, 250);
}


static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc) {
	char* buffer;
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	buffer = vtg_symbol_completion_provider_get_document_text (self, doc, self->priv->all_doc);
	vsc_source_buffer_set_source (self->priv->_sb, buffer);
	vsc_parser_manager_reparse_source_buffers (vsc_symbol_completion_get_parser (self->priv->_completion));
	buffer = (g_free (buffer), NULL);
}


static void vtg_symbol_completion_provider_real_finish (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	GList* _tmp0;
	self = (VtgSymbolCompletionProvider*) base;
	_tmp0 = NULL;
	self->priv->list = (_tmp0 = NULL, (self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL)), _tmp0);
}


static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	self = (VtgSymbolCompletionProvider*) base;
	return "SymbolCompletionProvider";
}


static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger) {
	VtgSymbolCompletionProvider * self;
	GTimer* timer;
	VscSymbolCompletionResult* _tmp0;
	gboolean _tmp1;
	GList* _tmp5;
	GList* _tmp6;
	self = (VtgSymbolCompletionProvider*) base;
	g_return_val_if_fail (trigger != NULL, NULL);
	timer = g_timer_new ();
	_tmp0 = NULL;
	vtg_symbol_completion_provider_transform_result (self, _tmp0 = vtg_symbol_completion_provider_find_proposals (self));
	(_tmp0 == NULL) ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL));
	g_timer_stop (timer);
	g_debug ("vtgsymbolcompletionprovider.vala:261: TOTAL TIME ELAPSED: %f", g_timer_elapsed (timer, NULL));
	_tmp1 = FALSE;
	if (g_list_length (self->priv->list) == 0) {
		_tmp1 = self->priv->cache_building;
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		VtgSymbolCompletionTrigger* _tmp3;
		VtgSymbolCompletionTrigger* _tmp2;
		_tmp3 = NULL;
		_tmp2 = NULL;
		self->priv->_last_trigger = (_tmp3 = (_tmp2 = VTG_SYMBOL_COMPLETION_TRIGGER (trigger), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL)), _tmp3);
	} else {
		VtgSymbolCompletionTrigger* _tmp4;
		_tmp4 = NULL;
		self->priv->_last_trigger = (_tmp4 = NULL, (self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL)), _tmp4);
	}
	_tmp5 = NULL;
	_tmp6 = NULL;
	return (_tmp6 = (_tmp5 = self->priv->list, self->priv->list = NULL, _tmp5), (timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL)), _tmp6);
}


static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b) {
	return vtg_symbol_completion_provider_proposal_sort (a, b);
}


static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon) {
	GscProposal** _tmp1;
	gint proposals_length1;
	gint _tmp0;
	GscProposal** proposals;
	g_return_if_fail (self != NULL);
	g_return_if_fail (symbols != NULL);
	g_return_if_fail (icon != NULL);
	_tmp1 = NULL;
	proposals = (_tmp1 = vtg_utils_get_proposal_cache (&_tmp0), proposals_length1 = _tmp0, _tmp1);
	{
		GeeIterator* _symbol_it;
		_symbol_it = gee_iterable_iterator ((GeeIterable*) symbols);
		while (gee_iterator_next (_symbol_it)) {
			VscSymbolCompletionItem* symbol;
			GscProposal* proposal;
			const char* _tmp2;
			const char* _tmp3;
			char* name;
			const char* _tmp4;
			const char* _tmp5;
			char* info;
			GscProposal* _tmp9;
			symbol = (VscSymbolCompletionItem*) gee_iterator_get (_symbol_it);
			proposal = NULL;
			_tmp2 = NULL;
			if (symbol->name != NULL) {
				_tmp2 = symbol->name;
			} else {
				_tmp2 = "<null>";
			}
			_tmp3 = NULL;
			name = (_tmp3 = (_tmp2), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
			_tmp4 = NULL;
			if (symbol->info != NULL) {
				_tmp4 = symbol->info;
			} else {
				_tmp4 = "";
			}
			_tmp5 = NULL;
			info = (_tmp5 = (_tmp4), (_tmp5 == NULL) ? NULL : g_strdup (_tmp5));
			if (self->priv->prealloc_index < VTG_UTILS_prealloc_count) {
				GscProposal* _tmp7;
				GscProposal* _tmp6;
				_tmp7 = NULL;
				_tmp6 = NULL;
				proposal = (_tmp7 = (_tmp6 = proposals[self->priv->prealloc_index], (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL)), _tmp7);
				self->priv->prealloc_index++;
				g_object_set (proposal, "label", name, NULL);
				g_object_set (proposal, "info", info, NULL);
				g_object_set (proposal, "icon", icon, NULL);
			} else {
				GscProposal* _tmp8;
				_tmp8 = NULL;
				proposal = (_tmp8 = gsc_proposal_new (name, info, icon), (proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL)), _tmp8);
			}
			_tmp9 = NULL;
			self->priv->list = g_list_append (self->priv->list, (_tmp9 = proposal, (_tmp9 == NULL) ? NULL : g_object_ref (_tmp9)));
			(symbol == NULL) ? NULL : (symbol = (g_object_unref (symbol), NULL));
			(proposal == NULL) ? NULL : (proposal = (g_object_unref (proposal), NULL));
			name = (g_free (name), NULL);
			info = (g_free (info), NULL);
		}
		(_symbol_it == NULL) ? NULL : (_symbol_it = (gee_collection_object_unref (_symbol_it), NULL));
	}
	/*sort list*/
	self->priv->list = g_list_sort (self->priv->list, _vtg_symbol_completion_provider_proposal_sort_gcompare_func);
}


static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b) {
	GscProposal* _tmp0;
	GscProposal* pa;
	GscProposal* _tmp1;
	GscProposal* pb;
	gint _tmp2;
	_tmp0 = NULL;
	pa = (_tmp0 = GSC_PROPOSAL (a), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	_tmp1 = NULL;
	pb = (_tmp1 = GSC_PROPOSAL (b), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	return (_tmp2 = strcmp (gsc_proposal_get_label (pa), gsc_proposal_get_label (pb)), (pa == NULL) ? NULL : (pa = (g_object_unref (pa), NULL)), (pb == NULL) ? NULL : (pb = (g_object_unref (pb), NULL)), _tmp2);
}


static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* result) {
	GTimer* timer;
	GList* _tmp0;
	gboolean _tmp1;
	g_return_if_fail (self != NULL);
	timer = g_timer_new ();
	self->priv->prealloc_index = 0;
	_tmp0 = NULL;
	self->priv->list = (_tmp0 = NULL, (self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL)), _tmp0);
	_tmp1 = FALSE;
	if (result != NULL) {
		_tmp1 = !vsc_symbol_completion_result_get_is_empty (result);
	} else {
		_tmp1 = FALSE;
	}
	if (_tmp1) {
		if (gee_collection_get_size ((GeeCollection*) result->fields) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->fields, self->priv->_icon_field);
		}
		if (gee_collection_get_size ((GeeCollection*) result->properties) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->properties, self->priv->_icon_property);
		}
		if (gee_collection_get_size ((GeeCollection*) result->methods) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->methods, self->priv->_icon_method);
		}
		if (gee_collection_get_size ((GeeCollection*) result->signals) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->signals, self->priv->_icon_signal);
		}
		if (gee_collection_get_size ((GeeCollection*) result->classes) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->classes, self->priv->_icon_class);
		}
		if (gee_collection_get_size ((GeeCollection*) result->interfaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->interfaces, self->priv->_icon_iface);
		}
		if (gee_collection_get_size ((GeeCollection*) result->structs) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->structs, self->priv->_icon_struct);
		}
		if (gee_collection_get_size ((GeeCollection*) result->enums) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->enums, self->priv->_icon_enum);
		}
		if (gee_collection_get_size ((GeeCollection*) result->constants) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->constants, self->priv->_icon_const);
		}
		if (gee_collection_get_size ((GeeCollection*) result->namespaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->namespaces, self->priv->_icon_namespace);
		}
		if (gee_collection_get_size ((GeeCollection*) result->others) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, result->others, self->priv->_icon_generic);
		}
	}
	g_timer_stop (timer);
	g_debug ("vtgsymbolcompletionprovider.vala:346:      TRANSFORM TOTAL TIME ELAPSED: %f", g_timer_elapsed (timer, NULL));
	(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
}


static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** word, char** line, gint* lineno, gint* colno) {
	GString* tmp;
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* _tmp0;
	char* _tmp1;
	char* _tmp3;
	const char* _tmp2;
	g_return_if_fail (self != NULL);
	*word = NULL;
	*line = NULL;
	tmp = g_string_new ("");
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	(*lineno) = gtk_text_iter_get_line (&start);
	(*colno) = gtk_text_iter_get_line_offset (&start);
	_tmp0 = NULL;
	(*word) = (_tmp0 = g_strdup (""), (*word) = (g_free ((*word)), NULL), _tmp0);
	_tmp1 = NULL;
	(*line) = (_tmp1 = g_strdup (""), (*line) = (g_free ((*line)), NULL), _tmp1);
	end = start;
	gtk_text_iter_set_line_offset (&start, 0);
	_tmp3 = NULL;
	_tmp2 = NULL;
	(*line) = (_tmp3 = (_tmp2 = gtk_text_iter_get_text (&start, &end), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), (*line) = (g_free ((*line)), NULL), _tmp3);
	if ((*colno) > 1) {
		char* _tmp7;
		start = end;
		while (TRUE) {
			gunichar ch;
			gboolean _tmp4;
			gtk_text_iter_backward_char (&start);
			if (gtk_text_iter_starts_line (&start)) {
				break;
			}
			ch = gtk_text_iter_get_char (&start);
			_tmp4 = FALSE;
			if (skip_leading_spaces) {
				_tmp4 = ch == ' ';
			} else {
				_tmp4 = FALSE;
			}
			if (_tmp4) {
				while (TRUE) {
					gtk_text_iter_backward_char (&start);
					if (gtk_text_iter_starts_line (&start)) {
						break;
					}
					ch = gtk_text_iter_get_char (&start);
					if (ch != ' ') {
						break;
					}
				}
				gtk_text_iter_forward_char (&start);
			} else {
				if (ch == ')') {
					while (TRUE) {
						gtk_text_iter_backward_char (&start);
						if (gtk_text_iter_starts_line (&start)) {
							break;
						}
						ch = gtk_text_iter_get_char (&start);
						if (ch == '(') {
							/*tmp.append_unichar (ch);*/
							break;
						}
					}
					/*skip spaces*/
					if (!gtk_text_iter_starts_line (&start)) {
						while (TRUE) {
							gtk_text_iter_backward_char (&start);
							if (gtk_text_iter_starts_line (&start)) {
								break;
							}
							ch = gtk_text_iter_get_char (&start);
							if (ch != ' ') {
								break;
							}
						}
					}
					if (!gtk_text_iter_starts_line (&start)) {
						g_string_append_unichar (tmp, ch);
					}
				} else {
					if (ch == '\"') {
						/*find where the string literal begin*/
						g_string_append_unichar (tmp, ch);
						while (TRUE) {
							gtk_text_iter_backward_char (&start);
							if (gtk_text_iter_starts_line (&start)) {
								break;
							}
							ch = gtk_text_iter_get_char (&start);
							g_string_append_unichar (tmp, ch);
							if (ch == '\"') {
								break;
							}
						}
					} else {
						gboolean _tmp5;
						gboolean _tmp6;
						_tmp5 = FALSE;
						_tmp6 = FALSE;
						if (g_unichar_isalnum (ch)) {
							_tmp6 = TRUE;
						} else {
							_tmp6 = ch == '.';
						}
						if (_tmp6) {
							_tmp5 = TRUE;
						} else {
							_tmp5 = ch == '_';
						}
						if (!(_tmp5)) {
							break;
						} else {
							g_string_append_unichar (tmp, ch);
						}
					}
				}
			}
			skip_leading_spaces = FALSE;
		}
		_tmp7 = NULL;
		(*word) = (_tmp7 = g_utf8_strreverse (tmp->str, -1), (*word) = (g_free ((*word)), NULL), _tmp7);
	}
	(tmp == NULL) ? NULL : (tmp = (g_string_free (tmp, TRUE), NULL));
}


static VscSymbolCompletionItem* vtg_symbol_completion_provider_find_method_signature (VtgSymbolCompletionProvider* self) {
	GError * inner_error;
	VscSymbolCompletionResult* result;
	char* line;
	char* word;
	gint lineno;
	gint colno;
	char* _tmp3;
	char* _tmp2;
	char* _tmp1;
	char* _tmp0;
	gboolean _tmp4;
	char** _tmp6;
	gint tmp_length1;
	char** tmp;
	gint count;
	char* first_part;
	const char* _tmp10;
	char* method;
	gboolean _tmp20;
	VscSymbolCompletionItem* _tmp22;
	g_return_val_if_fail (self != NULL, NULL);
	inner_error = NULL;
	result = NULL;
	line = NULL;
	word = NULL;
	lineno = 0;
	colno = 0;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	(vtg_symbol_completion_provider_parse_current_line (self, TRUE, &_tmp0, &_tmp2, &lineno, &colno), word = (_tmp1 = _tmp0, word = (g_free (word), NULL), _tmp1));
	line = (_tmp3 = _tmp2, line = (g_free (line), NULL), _tmp3);
	_tmp4 = FALSE;
	if (word == NULL) {
		_tmp4 = TRUE;
	} else {
		_tmp4 = _vala_strcmp0 (word, "") == 0;
	}
	if (_tmp4) {
		VscSymbolCompletionItem* _tmp5;
		_tmp5 = NULL;
		return (_tmp5 = NULL, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), line = (g_free (line), NULL), word = (g_free (word), NULL), _tmp5);
	}
	/* 
	  strip last type part. 
	  eg. for demos.demo.demo_method obtains
	  demos.demo + demo_method
	*/
	_tmp6 = NULL;
	tmp = (_tmp6 = g_strsplit (word, ".", 0), tmp_length1 = -1, _tmp6);
	count = 0;
	first_part = NULL;
	while (tmp[count] != NULL) {
		if (tmp[count + 1] != NULL) {
			if (first_part == NULL) {
				char* _tmp8;
				const char* _tmp7;
				_tmp8 = NULL;
				_tmp7 = NULL;
				first_part = (_tmp8 = (_tmp7 = tmp[count], (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), first_part = (g_free (first_part), NULL), _tmp8);
			} else {
				char* _tmp9;
				_tmp9 = NULL;
				first_part = (_tmp9 = g_strconcat (first_part, ".", tmp[count], NULL), first_part = (g_free (first_part), NULL), _tmp9);
			}
		}
		count++;
	}
	_tmp10 = NULL;
	method = (_tmp10 = tmp[count - 1], (_tmp10 == NULL) ? NULL : g_strdup (_tmp10));
	if (first_part == NULL) {
		char* _tmp12;
		const char* _tmp11;
		_tmp12 = NULL;
		_tmp11 = NULL;
		first_part = (_tmp12 = (_tmp11 = method, (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), first_part = (g_free (first_part), NULL), _tmp12);
	}
	{
		char* typename;
		ValaDataType* dt;
		VscSymbolCompletionFilterOptions* options;
		gboolean _tmp14;
		typename = NULL;
		dt = vsc_symbol_completion_get_datatype_for_name (self->priv->_completion, first_part, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
		if (inner_error != NULL) {
			goto __catch0_g_error;
		}
		if (dt != NULL) {
			char* _tmp13;
			_tmp13 = NULL;
			typename = (_tmp13 = vsc_symbol_completion_get_qualified_name_for_datatype (self->priv->_completion, dt), typename = (g_free (typename), NULL), _tmp13);
		}
		options = vsc_symbol_completion_filter_options_new ();
		vsc_symbol_completion_filter_options_public_only (options);
		_tmp14 = FALSE;
		if (strstr (line, "= new ") != NULL) {
			_tmp14 = TRUE;
		} else {
			_tmp14 = strstr (line, "=new ") != NULL;
		}
		if (_tmp14) {
			options->only_constructors = TRUE;
		}
		if (typename != NULL) {
			VscSymbolCompletionResult* _tmp15;
			VscSymbolCompletionResult* _tmp16;
			g_debug ("vtgsymbolcompletionprovider.vala:491: datatype '%s' for: %s", typename, word);
			options->static_symbols = FALSE;
			if (_vala_strcmp0 (word, "this") == 0) {
				options->private_symbols = TRUE;
				options->protected_symbols = TRUE;
			} else {
				if (_vala_strcmp0 (word, "base") == 0) {
					options->protected_symbols = TRUE;
				}
			}
			_tmp15 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), &inner_error);
			if (inner_error != NULL) {
				goto __catch0_g_error;
			}
			_tmp16 = NULL;
			result = (_tmp16 = _tmp15, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp16);
		} else {
			gboolean _tmp17;
			_tmp17 = FALSE;
			if (g_str_has_prefix (first_part, "this.") == FALSE) {
				_tmp17 = g_str_has_prefix (first_part, "base.") == FALSE;
			} else {
				_tmp17 = FALSE;
			}
			if (_tmp17) {
				VscSymbolCompletionResult* _tmp18;
				VscSymbolCompletionResult* _tmp19;
				options->static_symbols = TRUE;
				options->interface_symbols = FALSE;
				_tmp18 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, first_part, vsc_source_buffer_get_name (self->priv->_sb), &inner_error);
				if (inner_error != NULL) {
					goto __catch0_g_error;
				}
				_tmp19 = NULL;
				result = (_tmp19 = _tmp18, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp19);
			}
		}
		typename = (g_free (typename), NULL);
		(dt == NULL) ? NULL : (dt = (vala_code_node_unref (dt), NULL));
		(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgsymbolcompletionprovider.vala:509: error: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally0:
	;
	_tmp20 = FALSE;
	if (result != NULL) {
		_tmp20 = gee_collection_get_size ((GeeCollection*) result->methods) > 0;
	} else {
		_tmp20 = FALSE;
	}
	if (_tmp20) {
		{
			GeeIterator* _item_it;
			_item_it = gee_iterable_iterator ((GeeIterable*) result->methods);
			while (gee_iterator_next (_item_it)) {
				VscSymbolCompletionItem* item;
				item = (VscSymbolCompletionItem*) gee_iterator_get (_item_it);
				if (_vala_strcmp0 (item->name, method) == 0) {
					VscSymbolCompletionItem* _tmp21;
					_tmp21 = NULL;
					return (_tmp21 = item, (_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL)), (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), line = (g_free (line), NULL), word = (g_free (word), NULL), tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL), first_part = (g_free (first_part), NULL), method = (g_free (method), NULL), _tmp21);
				}
				(item == NULL) ? NULL : (item = (g_object_unref (item), NULL));
			}
			(_item_it == NULL) ? NULL : (_item_it = (gee_collection_object_unref (_item_it), NULL));
		}
	}
	_tmp22 = NULL;
	return (_tmp22 = NULL, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), line = (g_free (line), NULL), word = (g_free (word), NULL), tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL), first_part = (g_free (first_part), NULL), method = (g_free (method), NULL), _tmp22);
}


static VscSymbolCompletionResult* vtg_symbol_completion_provider_find_proposals (VtgSymbolCompletionProvider* self) {
	GError * inner_error;
	VscSymbolCompletionResult* result;
	char* line;
	char* word;
	gint lineno;
	gint colno;
	char* _tmp3;
	char* _tmp2;
	char* _tmp1;
	char* _tmp0;
	gboolean _tmp4;
	VscSymbolCompletionResult* _tmp18;
	g_return_val_if_fail (self != NULL, NULL);
	inner_error = NULL;
	result = NULL;
	line = NULL;
	word = NULL;
	lineno = 0;
	colno = 0;
	_tmp3 = NULL;
	_tmp2 = NULL;
	_tmp1 = NULL;
	_tmp0 = NULL;
	(vtg_symbol_completion_provider_parse_current_line (self, FALSE, &_tmp0, &_tmp2, &lineno, &colno), word = (_tmp1 = _tmp0, word = (g_free (word), NULL), _tmp1));
	line = (_tmp3 = _tmp2, line = (g_free (line), NULL), _tmp3);
	_tmp4 = FALSE;
	if (word == NULL) {
		_tmp4 = _vala_strcmp0 (word, "") == 0;
	} else {
		_tmp4 = FALSE;
	}
	if (_tmp4) {
		VscSymbolCompletionResult* _tmp5;
		_tmp5 = NULL;
		return (_tmp5 = NULL, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), line = (g_free (line), NULL), word = (g_free (word), NULL), _tmp5);
	}
	{
		char* typename;
		GTimer* timer;
		gboolean _tmp6;
		VscSymbolCompletionFilterOptions* options;
		gboolean _tmp10;
		typename = NULL;
		g_debug ("vtgsymbolcompletionprovider.vala:534: (find_proposals): START search datatype for: '%s'", word);
		timer = g_timer_new ();
		_tmp6 = FALSE;
		if (g_str_has_prefix (word, "\"")) {
			_tmp6 = g_str_has_suffix (word, "\"");
		} else {
			_tmp6 = FALSE;
		}
		if (_tmp6) {
			char* _tmp7;
			_tmp7 = NULL;
			typename = (_tmp7 = g_strdup ("string"), typename = (g_free (typename), NULL), _tmp7);
		} else {
			char* _tmp8;
			char* _tmp9;
			_tmp8 = vsc_symbol_completion_get_datatype_name_for_name (self->priv->_completion, word, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &inner_error);
			if (inner_error != NULL) {
				goto __catch1_g_error;
			}
			_tmp9 = NULL;
			typename = (_tmp9 = _tmp8, typename = (g_free (typename), NULL), _tmp9);
		}
		g_debug ("vtgsymbolcompletionprovider.vala:541: (find_proposals): END search datatype for: '%s'", word);
		options = vsc_symbol_completion_filter_options_new ();
		vsc_symbol_completion_filter_options_public_only (options);
		_tmp10 = FALSE;
		if (strstr (line, "= new ") != NULL) {
			_tmp10 = TRUE;
		} else {
			_tmp10 = strstr (line, "=new ") != NULL;
		}
		if (_tmp10) {
			options->only_constructors = TRUE;
		}
		if (typename != NULL) {
			char* _tmp12;
			const char* _tmp11;
			VscSymbolCompletionResult* _tmp13;
			VscSymbolCompletionResult* _tmp14;
			g_debug ("vtgsymbolcompletionprovider.vala:548: datatype '%s' for: %s", typename, word);
			options->static_symbols = FALSE;
			if (_vala_strcmp0 (word, "this") == 0) {
				options->private_symbols = TRUE;
				options->protected_symbols = TRUE;
			} else {
				if (_vala_strcmp0 (word, "base") == 0) {
					options->protected_symbols = TRUE;
				}
			}
			_tmp12 = NULL;
			_tmp11 = NULL;
			options->exclude_type = (_tmp12 = (_tmp11 = typename, (_tmp11 == NULL) ? NULL : g_strdup (_tmp11)), options->exclude_type = (g_free (options->exclude_type), NULL), _tmp12);
			g_debug ("vtgsymbolcompletionprovider.vala:557: (find_proposals): START completion for: '%s'", typename);
			g_timer_start (timer);
			_tmp13 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), &inner_error);
			if (inner_error != NULL) {
				goto __catch1_g_error;
			}
			_tmp14 = NULL;
			result = (_tmp14 = _tmp13, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp14);
			g_timer_stop (timer);
			g_debug ("vtgsymbolcompletionprovider.vala:561: (find_proposals): END completion for: '%s', Count %d, TIME Elapsed: %f", typename, vsc_symbol_completion_result_get_count (result), g_timer_elapsed (timer, NULL));
		} else {
			gboolean _tmp15;
			g_debug ("vtgsymbolcompletionprovider.vala:563: data type not found for: %s", word);
			_tmp15 = FALSE;
			if (g_str_has_prefix (word, "this.") == FALSE) {
				_tmp15 = g_str_has_prefix (word, "base.") == FALSE;
			} else {
				_tmp15 = FALSE;
			}
			if (_tmp15) {
				VscSymbolCompletionResult* _tmp16;
				VscSymbolCompletionResult* _tmp17;
				options->static_symbols = TRUE;
				options->interface_symbols = FALSE;
				g_debug ("vtgsymbolcompletionprovider.vala:567: (find_proposals): START STATIC completion for: '%s'", word);
				g_timer_start (timer);
				_tmp16 = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, word, vsc_source_buffer_get_name (self->priv->_sb), &inner_error);
				if (inner_error != NULL) {
					goto __catch1_g_error;
				}
				_tmp17 = NULL;
				result = (_tmp17 = _tmp16, (result == NULL) ? NULL : (result = (g_object_unref (result), NULL)), _tmp17);
				g_timer_stop (timer);
				g_debug ("vtgsymbolcompletionprovider.vala:571: (find_proposals): END STATIC completion for: '%s', Count %d, TIME Elapsed: %f", word, vsc_symbol_completion_result_get_count (result), g_timer_elapsed (timer, NULL));
			}
		}
		typename = (g_free (typename), NULL);
		(timer == NULL) ? NULL : (timer = (g_timer_destroy (timer), NULL));
		(options == NULL) ? NULL : (options = (vsc_symbol_completion_filter_options_unref (options), NULL));
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgsymbolcompletionprovider.vala:575: %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally1:
	;
	_tmp18 = NULL;
	return (_tmp18 = result, line = (g_free (line), NULL), word = (g_free (word), NULL), _tmp18);
}


static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc) {
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* doc_text;
	gboolean _tmp0;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (doc != NULL, NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	doc_text = NULL;
	_tmp0 = FALSE;
	if (all_doc) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = gedit_document_is_untouched (doc);
	}
	if (_tmp0) {
		char* _tmp2;
		const char* _tmp1;
		end = start;
		gtk_text_iter_set_line_offset (&start, 0);
		while (gtk_text_iter_backward_line (&start)) {
			;
		}
		while (gtk_text_iter_forward_line (&end)) {
			;
		}
		_tmp2 = NULL;
		_tmp1 = NULL;
		doc_text = (_tmp2 = (_tmp1 = gtk_text_iter_get_text (&start, &end), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1)), doc_text = (g_free (doc_text), NULL), _tmp2);
	} else {
		const char* _tmp3;
		char* text1;
		char* text2;
		char* _tmp6;
		end = start;
		gtk_text_iter_set_line_offset (&end, 0);
		while (gtk_text_iter_backward_line (&start)) {
			;
		}
		_tmp3 = NULL;
		text1 = (_tmp3 = gtk_text_iter_get_text (&start, &end), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
		text2 = g_strdup ("");
		/*trick: jump the current edited row (there
		are a lot of probability that this row will
		cause a parser error)*/
		if (gtk_text_iter_forward_line (&end)) {
			char* _tmp5;
			const char* _tmp4;
			start = end;
			while (gtk_text_iter_forward_line (&end)) {
				;
			}
			_tmp5 = NULL;
			_tmp4 = NULL;
			text2 = (_tmp5 = (_tmp4 = gtk_text_iter_get_text (&start, &end), (_tmp4 == NULL) ? NULL : g_strdup (_tmp4)), text2 = (g_free (text2), NULL), _tmp5);
		}
		_tmp6 = NULL;
		doc_text = (_tmp6 = g_strdup_printf ("%s\n%s", text1, text2), doc_text = (g_free (doc_text), NULL), _tmp6);
		text1 = (g_free (text1), NULL);
		text2 = (g_free (text2), NULL);
	}
	return doc_text;
}


static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value) {
	VscSymbolCompletion* _tmp1;
	VscSymbolCompletion* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_completion = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "completion");
}


static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value) {
	GeditView* _tmp1;
	GeditView* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_view = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "view");
}


static void vtg_symbol_completion_provider_set_plugin (VtgSymbolCompletionProvider* self, VtgPlugin* value) {
	VtgPlugin* _tmp1;
	VtgPlugin* _tmp0;
	g_return_if_fail (self != NULL);
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_plugin = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp1);
	g_object_notify ((GObject *) self, "plugin");
}


static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, GdkEventKey* event, gpointer self) {
	return vtg_symbol_completion_provider_on_view_key_press (self, _sender, event);
}


static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_idle (self);
}


static void __lambda0 (VscParserManager* sender, VtgSymbolCompletionProvider* self) {
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == FALSE) {
		self->priv->cache_building = TRUE;
		g_idle_add (_vtg_symbol_completion_provider_on_idle_gsource_func, self);
	}
}


static void ___lambda0_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self) {
	__lambda0 (_sender, self);
}


static void __lambda1 (VscParserManager* sender, VtgSymbolCompletionProvider* self) {
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == TRUE) {
		self->priv->cache_building = FALSE;
		g_idle_add (_vtg_symbol_completion_provider_on_idle_gsource_func, self);
	}
}


static void ___lambda1_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self) {
	__lambda1 (_sender, self);
}


static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgSymbolCompletionProviderClass * klass;
	GObjectClass * parent_class;
	VtgSymbolCompletionProvider * self;
	GError * inner_error;
	klass = VTG_SYMBOL_COMPLETION_PROVIDER_CLASS (g_type_class_peek (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	inner_error = NULL;
	{
		{
			GeditDocument* _tmp0;
			GeditDocument* doc;
			const char* _tmp1;
			char* name;
			VscSourceBuffer* _tmp4;
			GdkPixbuf* _tmp5;
			GdkPixbuf* _tmp7;
			GdkPixbuf* _tmp6;
			GdkPixbuf* _tmp9;
			char* _tmp8;
			GdkPixbuf* _tmp11;
			char* _tmp10;
			GdkPixbuf* _tmp13;
			char* _tmp12;
			GdkPixbuf* _tmp15;
			char* _tmp14;
			GdkPixbuf* _tmp17;
			char* _tmp16;
			GdkPixbuf* _tmp19;
			char* _tmp18;
			GdkPixbuf* _tmp21;
			char* _tmp20;
			GdkPixbuf* _tmp23;
			char* _tmp22;
			GdkPixbuf* _tmp25;
			char* _tmp24;
			GdkPixbuf* _tmp27;
			char* _tmp26;
			GeditStatusbar* _tmp28;
			GeditStatusbar* status_bar;
			_tmp0 = NULL;
			doc = (_tmp0 = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
			_tmp1 = NULL;
			name = (_tmp1 = gedit_document_get_uri (doc), (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
			if (name == NULL) {
				char* _tmp3;
				const char* _tmp2;
				_tmp3 = NULL;
				_tmp2 = NULL;
				name = (_tmp3 = (_tmp2 = gedit_document_get_short_name_for_display (doc), (_tmp2 == NULL) ? NULL : g_strdup (_tmp2)), name = (g_free (name), NULL), _tmp3);
			}
			_tmp4 = NULL;
			self->priv->_sb = (_tmp4 = vsc_source_buffer_new (name, NULL), (self->priv->_sb == NULL) ? NULL : (self->priv->_sb = (g_object_unref (self->priv->_sb), NULL)), _tmp4);
			vsc_source_buffer_set_data (self->priv->_sb, (void*) doc);
			g_signal_connect_object ((GtkWidget*) self->priv->_view, "key-press-event", (GCallback) _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event, self, 0);
			vsc_parser_manager_add_source_buffer (vsc_symbol_completion_get_parser (self->priv->_completion), self->priv->_sb, &inner_error);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp5 = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &inner_error);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp7 = NULL;
			_tmp6 = NULL;
			self->priv->_icon_generic = (_tmp7 = (_tmp6 = _tmp5, (_tmp6 == NULL) ? NULL : g_object_ref (_tmp6)), (self->priv->_icon_generic == NULL) ? NULL : (self->priv->_icon_generic = (g_object_unref (self->priv->_icon_generic), NULL)), _tmp7);
			_tmp9 = NULL;
			_tmp8 = NULL;
			self->priv->_icon_field = (_tmp9 = gdk_pixbuf_new_from_file (_tmp8 = vtg_utils_get_image_path ("element-field-16.png"), &inner_error), (self->priv->_icon_field == NULL) ? NULL : (self->priv->_icon_field = (g_object_unref (self->priv->_icon_field), NULL)), _tmp9);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp8 = (g_free (_tmp8), NULL);
			_tmp11 = NULL;
			_tmp10 = NULL;
			self->priv->_icon_method = (_tmp11 = gdk_pixbuf_new_from_file (_tmp10 = vtg_utils_get_image_path ("element-method-16.png"), &inner_error), (self->priv->_icon_method == NULL) ? NULL : (self->priv->_icon_method = (g_object_unref (self->priv->_icon_method), NULL)), _tmp11);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp10 = (g_free (_tmp10), NULL);
			_tmp13 = NULL;
			_tmp12 = NULL;
			self->priv->_icon_class = (_tmp13 = gdk_pixbuf_new_from_file (_tmp12 = vtg_utils_get_image_path ("element-class-16.png"), &inner_error), (self->priv->_icon_class == NULL) ? NULL : (self->priv->_icon_class = (g_object_unref (self->priv->_icon_class), NULL)), _tmp13);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp12 = (g_free (_tmp12), NULL);
			_tmp15 = NULL;
			_tmp14 = NULL;
			self->priv->_icon_struct = (_tmp15 = gdk_pixbuf_new_from_file (_tmp14 = vtg_utils_get_image_path ("element-structure-16.png"), &inner_error), (self->priv->_icon_struct == NULL) ? NULL : (self->priv->_icon_struct = (g_object_unref (self->priv->_icon_struct), NULL)), _tmp15);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp14 = (g_free (_tmp14), NULL);
			_tmp17 = NULL;
			_tmp16 = NULL;
			self->priv->_icon_property = (_tmp17 = gdk_pixbuf_new_from_file (_tmp16 = vtg_utils_get_image_path ("element-property-16.png"), &inner_error), (self->priv->_icon_property == NULL) ? NULL : (self->priv->_icon_property = (g_object_unref (self->priv->_icon_property), NULL)), _tmp17);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp16 = (g_free (_tmp16), NULL);
			_tmp19 = NULL;
			_tmp18 = NULL;
			self->priv->_icon_signal = (_tmp19 = gdk_pixbuf_new_from_file (_tmp18 = vtg_utils_get_image_path ("element-event-16.png"), &inner_error), (self->priv->_icon_signal == NULL) ? NULL : (self->priv->_icon_signal = (g_object_unref (self->priv->_icon_signal), NULL)), _tmp19);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp18 = (g_free (_tmp18), NULL);
			_tmp21 = NULL;
			_tmp20 = NULL;
			self->priv->_icon_iface = (_tmp21 = gdk_pixbuf_new_from_file (_tmp20 = vtg_utils_get_image_path ("element-interface-16.png"), &inner_error), (self->priv->_icon_iface == NULL) ? NULL : (self->priv->_icon_iface = (g_object_unref (self->priv->_icon_iface), NULL)), _tmp21);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp20 = (g_free (_tmp20), NULL);
			_tmp23 = NULL;
			_tmp22 = NULL;
			self->priv->_icon_enum = (_tmp23 = gdk_pixbuf_new_from_file (_tmp22 = vtg_utils_get_image_path ("element-enumeration-16.png"), &inner_error), (self->priv->_icon_enum == NULL) ? NULL : (self->priv->_icon_enum = (g_object_unref (self->priv->_icon_enum), NULL)), _tmp23);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp22 = (g_free (_tmp22), NULL);
			_tmp25 = NULL;
			_tmp24 = NULL;
			self->priv->_icon_const = (_tmp25 = gdk_pixbuf_new_from_file (_tmp24 = vtg_utils_get_image_path ("element-literal-16.png"), &inner_error), (self->priv->_icon_const == NULL) ? NULL : (self->priv->_icon_const = (g_object_unref (self->priv->_icon_const), NULL)), _tmp25);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp24 = (g_free (_tmp24), NULL);
			_tmp27 = NULL;
			_tmp26 = NULL;
			self->priv->_icon_namespace = (_tmp27 = gdk_pixbuf_new_from_file (_tmp26 = vtg_utils_get_image_path ("element-namespace-16.png"), &inner_error), (self->priv->_icon_namespace == NULL) ? NULL : (self->priv->_icon_namespace = (g_object_unref (self->priv->_icon_namespace), NULL)), _tmp27);
			if (inner_error != NULL) {
				goto __catch2_g_error;
			}
			_tmp26 = (g_free (_tmp26), NULL);
			g_signal_connect (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-building", (GCallback) ___lambda0_vsc_parser_manager_cache_building, self);
			g_signal_connect (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-builded", (GCallback) ___lambda1_vsc_parser_manager_cache_builded, self);
			_tmp28 = NULL;
			status_bar = (_tmp28 = GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_get_gedit_window (self->priv->_plugin))), (_tmp28 == NULL) ? NULL : g_object_ref (_tmp28));
			self->priv->sb_context_id = gtk_statusbar_get_context_id ((GtkStatusbar*) status_bar, "symbol status");
			self->priv->cache_building = TRUE;
			self->priv->all_doc = TRUE;
			vtg_symbol_completion_provider_parse (self, doc);
			(doc == NULL) ? NULL : (doc = (g_object_unref (doc), NULL));
			name = (g_free (name), NULL);
			(status_bar == NULL) ? NULL : (status_bar = (g_object_unref (status_bar), NULL));
		}
		goto __finally2;
		__catch2_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgsymbolcompletionprovider.vala:125: an error occourred: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally2:
		;
	}
	return obj;
}


static void vtg_symbol_completion_provider_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION:
		vtg_symbol_completion_provider_set_completion (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_VIEW:
		vtg_symbol_completion_provider_set_view (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN:
		vtg_symbol_completion_provider_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_symbol_completion_provider_class_init (VtgSymbolCompletionProviderClass * klass) {
	vtg_symbol_completion_provider_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSymbolCompletionProviderPrivate));
	G_OBJECT_CLASS (klass)->set_property = vtg_symbol_completion_provider_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_symbol_completion_provider_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_symbol_completion_provider_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION, g_param_spec_object ("completion", "completion", "completion", VSC_TYPE_SYMBOL_COMPLETION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_symbol_completion_provider_gsc_provider_interface_init (GscProviderIface * iface) {
	vtg_symbol_completion_provider_gsc_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->finish = vtg_symbol_completion_provider_real_finish;
	iface->get_name = vtg_symbol_completion_provider_real_get_name;
	iface->get_proposals = vtg_symbol_completion_provider_real_get_proposals;
}


static void vtg_symbol_completion_provider_instance_init (VtgSymbolCompletionProvider * self) {
	self->priv = VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE (self);
	self->priv->_sb = NULL;
	self->priv->_completion = NULL;
	self->priv->_view = NULL;
	self->priv->timeout_id = (guint) 0;
	self->priv->all_doc = FALSE;
	self->priv->prealloc_index = 0;
	self->priv->counter = 0;
	self->priv->cache_building = FALSE;
	self->priv->prev_cache_building = FALSE;
	self->priv->tooltip_is_visible = FALSE;
	self->priv->_last_trigger = NULL;
	self->priv->sb_msg_id = (guint) 0;
	self->priv->sb_context_id = (guint) 0;
	self->priv->_calltip_window = NULL;
	self->priv->need_parse = TRUE;
	self->priv->current_edited_line = -1;
}


static void vtg_symbol_completion_provider_finalize (GObject* obj) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	(self->priv->_sb == NULL) ? NULL : (self->priv->_sb = (g_object_unref (self->priv->_sb), NULL));
	(self->priv->_completion == NULL) ? NULL : (self->priv->_completion = (g_object_unref (self->priv->_completion), NULL));
	(self->priv->_view == NULL) ? NULL : (self->priv->_view = (g_object_unref (self->priv->_view), NULL));
	(self->priv->list == NULL) ? NULL : (self->priv->list = (_g_list_free_g_object_unref (self->priv->list), NULL));
	(self->priv->_icon_generic == NULL) ? NULL : (self->priv->_icon_generic = (g_object_unref (self->priv->_icon_generic), NULL));
	(self->priv->_icon_field == NULL) ? NULL : (self->priv->_icon_field = (g_object_unref (self->priv->_icon_field), NULL));
	(self->priv->_icon_method == NULL) ? NULL : (self->priv->_icon_method = (g_object_unref (self->priv->_icon_method), NULL));
	(self->priv->_icon_class == NULL) ? NULL : (self->priv->_icon_class = (g_object_unref (self->priv->_icon_class), NULL));
	(self->priv->_icon_struct == NULL) ? NULL : (self->priv->_icon_struct = (g_object_unref (self->priv->_icon_struct), NULL));
	(self->priv->_icon_property == NULL) ? NULL : (self->priv->_icon_property = (g_object_unref (self->priv->_icon_property), NULL));
	(self->priv->_icon_signal == NULL) ? NULL : (self->priv->_icon_signal = (g_object_unref (self->priv->_icon_signal), NULL));
	(self->priv->_icon_iface == NULL) ? NULL : (self->priv->_icon_iface = (g_object_unref (self->priv->_icon_iface), NULL));
	(self->priv->_icon_const == NULL) ? NULL : (self->priv->_icon_const = (g_object_unref (self->priv->_icon_const), NULL));
	(self->priv->_icon_enum == NULL) ? NULL : (self->priv->_icon_enum = (g_object_unref (self->priv->_icon_enum), NULL));
	(self->priv->_icon_namespace == NULL) ? NULL : (self->priv->_icon_namespace = (g_object_unref (self->priv->_icon_namespace), NULL));
	(self->priv->_last_trigger == NULL) ? NULL : (self->priv->_last_trigger = (g_object_unref (self->priv->_last_trigger), NULL));
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_calltip_window == NULL) ? NULL : (self->priv->_calltip_window = (g_object_unref (self->priv->_calltip_window), NULL));
	G_OBJECT_CLASS (vtg_symbol_completion_provider_parent_class)->finalize (obj);
}


static GType vtg_symbol_completion_provider_type_id = 0;
GType vtg_symbol_completion_provider_get_type (void) {
	return vtg_symbol_completion_provider_type_id;
}


GType vtg_symbol_completion_provider_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSymbolCompletionProviderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_symbol_completion_provider_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSymbolCompletionProvider), 0, (GInstanceInitFunc) vtg_symbol_completion_provider_instance_init, NULL };
	static const GInterfaceInfo gsc_provider_info = { (GInterfaceInitFunc) vtg_symbol_completion_provider_gsc_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_symbol_completion_provider_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSymbolCompletionProvider", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_symbol_completion_provider_type_id, GSC_TYPE_PROVIDER, &gsc_provider_info);
	return vtg_symbol_completion_provider_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL)
			destroy_func (((gpointer*) array)[i]);
		}
		else
		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) array)[i]);
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




